/**
 * section: Tree
 * synopsis: Navigates a tree to print element names
 * purpose: Parse a file to a tree, use xmlDocGetRootElement() to
 *          get the root element, then walk the document and print
 *          all the element name in document order.
 * usage: tree1 filename_or_URL
 * test: tree1 test2.xml > tree1.tmp ; diff tree1.tmp tree1.res ; rm tree1.tmp
 * author: Dodji Seketeli
 * copy: see Copyright for the status of this software.
 */

#include "sngtc_server.h"

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xmlmemory.h>

#ifdef LIBXML_TREE_ENABLED

static int host_nic_vocallo_idx=-1;


/*
 *To compile this file using gcc you can type
 *gcc `xml2-config --cflags --libs` -o xmlexample libxml2-example.c
 */

int parse_element(xmlNode *cur)
{
	int err;
	xmlChar *name, *value;
	xmlChar *mac;
	system_eth_cfg_t *eth_cfg=NULL;

	SNGTC_DEBUG(" cur->name = %s\n",cur->name);
	
	if (!xmlStrcmp(cur->name, (const xmlChar *) "vocallo")) {
		host_nic_vocallo_idx++;
		name=xmlGetProp(cur, (const xmlChar *) "name");
		err=sngtc_find_mac_by_eth(name,&eth_cfg);
		if (err) {
			SNGTC_ERROR("Error: failed to find eth dev %s\n",name);
			return -1;
		}
		SNGTC_INFO("Eth Dev %s mac %s\n",name,eth_cfg->mac.mac_str);

		sprintf((char*)g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].host_mac.mac_str,eth_cfg->mac.mac_str);

		sngtc_codec_ipv4_str_hex(eth_cfg->ip.ip_str, &g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].host_ip);
		sngtc_codec_ipv4_str_hex("255.255.255.0", &g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].host_ip_netmask);
		sngtc_codec_ipv4_str_hex("255.255.255.0", &g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].vocallo_ip_netmask);

		g_init.host_nic_vocallo_sz++;
		g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].vocallo_ip				=eth_cfg->ip.ip;
		g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].vocallo_ip 				&= 0xFFFFFF00;
		g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].vocallo_ip 				|= 0x00000064;
		g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].vocallo_base_udp_port 	= 10000;
		g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].silence_suppression 		= 0;
	}


	if (!xmlStrcmp(cur->name, (const xmlChar *) "param")) {
		name=xmlGetProp(cur, (const xmlChar *) "name");
		value=xmlGetProp(cur, (const xmlChar *) "value");
		if (name && value) {
			if (!xmlStrcmp(name, (const xmlChar *) "baseudp") && atoi((char*)value) > 0) {
				SNGTC_INFO("Vocallo [%i]:  baseudp %i\n",host_nic_vocallo_idx, atoi((char*)value));
				g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].vocallo_base_udp_port = atoi((char*)value);
			}
			if (!xmlStrcmp(name, (const xmlChar *) "vocalloaddr")) {
				SNGTC_INFO("Vocallo [%i]:  vocalloaddr %s\n",host_nic_vocallo_idx,value);
				sngtc_codec_ipv4_str_hex((char*)value, &g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].vocallo_ip);
			}
			if (!xmlStrcmp(name, (const xmlChar *) "vocallo_base_addr")) {

				SNGTC_INFO("Vocallo [%i]:  vocallo_base_addr %s\n",host_nic_vocallo_idx,value);

				g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].vocallo_ip &= 0xFFFFFF00;
				g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].vocallo_ip |= atoi((char*)value)&0xFF;
			}
			if (!xmlStrcmp(name, (const xmlChar *) "silence_suppression") && atoi((char*)value) > 0) {
				SNGTC_INFO("Vocallo [%i]:  silence_suppression %i\n",host_nic_vocallo_idx,atoi((char*)value));
				g_init.host_nic_vocallo_cfg[host_nic_vocallo_idx].silence_suppression = atoi((char*)value);
			}
		}
	}

	return 0;
}

/**
 * parse_element_names:
 * @a_node: the initial xml node to consider.
 *
 * Prints the names of the all the xml elements
 * that are siblings or children of a given xml node.
 */
static int parse_element_names(xmlNode * a_node)
{
    xmlNode *cur_node = NULL;
	int err=0;

    for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if (cur_node->type == XML_ELEMENT_NODE) {
			err=parse_element(cur_node);
			if (err) {
				return err;
			}
        }
		parse_element_names(cur_node->children);
    }

	return err;
}


/**
 * Simple example to parse a file called "file.xml",
 * walk down the DOM, and print the name of the
 * xml elements nodes.
 */
int sngtc_parse_xml_config_file(char *file)
{
    xmlDoc *doc = NULL;
    xmlNode *root_element = NULL;
	int err;

	err=sngtc_detect_eth_dev();
	if (err < 1) {
		return err;
	}

    /*
     * this initialize the library and check potential ABI mismatches
     * between the version it was compiled for and the actual shared
     * library used.
     */
    LIBXML_TEST_VERSION

    /*parse the file and get the DOM */
    doc = xmlReadFile(file, NULL, 0);
    if (doc == NULL) {
        SNGTC_ERROR("Error: could not parse file %s\n",file);
		return -1;
    }

    /*Get the root element node */
    root_element = xmlDocGetRootElement(doc);

    parse_element_names(root_element);

    /*free the document */
    xmlFreeDoc(doc);

    /*
     *Free the global variables that may
     *have been allocated by the parser.
     */
    xmlCleanupParser();

    return 0;
}
#else
#error "XML Tree Support not compiled in!\n"
#endif

